#include "version.h"
#include <fstream.h>
#include "linalg.h"
#include "stringo.h"
#include "xtalutil.h"
#include "parse.h"

void tracev(const Array<Real> &v) {
  for (int i=0; i<v.get_size(); i++) {
    cout << v(i) << " ";
  }
  cout << endl;
}

void tracevl(const Array<Real> &v) {
  for (int i=0; i<v.get_size(); i++) {
    cout << v(i) << " ";
  }
}

class OptimizerParam {
public: 
  Real gradtol;
  Real trialstep;
  Real maxstep;
  Real multline;
  int maxitline;
  int maxitcg;
  int badstrike;
  OptimizerParam(void) {
    gradtol=1e-3;
    trialstep=0.1;
    maxstep=0.4;
    multline=2.0;
    maxitline=50;
    maxitcg=50;
    badstrike=1;
  }
  OptimizerParam(const  OptimizerParam &a) {
    gradtol=a.gradtol;
    trialstep=a.trialstep;
    maxstep=a.maxstep;
    multline=a.multline;
    maxitline=a.maxitline;
    maxitcg=a.maxitcg;
    badstrike=a.badstrike;
  }
  void operator = (const  OptimizerParam &a) {
    gradtol=a.gradtol;
    trialstep=a.trialstep;
    maxstep=a.maxstep;
    multline=a.multline;
    maxitline=a.maxitline;
    maxitcg=a.maxitcg;
    badstrike=a.badstrike;
  }
};

class FunctionWithGrad {
public:
  virtual void set_arg(const Array<Real> &x) {}
  virtual Real get_val(void) const {}
  virtual void get_grad(Array<Real> *df) const {}
};

class Function1DWithGrad {
public: 
  virtual void set_arg(Real x) {}
  virtual Real get_val(void) const {}
  virtual Real get_grad(void) const {}
};

class FunctionAlongLine: public  Function1DWithGrad {
  Array<Real> org;
  Array<Real> dir;
  FunctionWithGrad *pfunc;
  Real val;
  Real grad;
public:
  FunctionAlongLine(const Array<Real> &_org, const Array<Real> &_dir,  FunctionWithGrad *_pfunc): Function1DWithGrad(), org(_org), dir(_dir) {
    pfunc=_pfunc;
    val=pfunc->get_val();
    Array<Real> vgrad;
    pfunc->get_grad(&vgrad);
    grad=inner_product(vgrad,dir);
  }
  virtual void scalar_to_vector(Array<Real> *px, Real u) {
    product(px,dir,u);
    sum(px,*px,org);
  }
  virtual void set_arg(Real u) {
    Array<Real> x;
    scalar_to_vector(&x,u);
    pfunc->set_arg(x);
    val=pfunc->get_val();
    Array<Real> vgrad;
    pfunc->get_grad(&vgrad);
    grad=inner_product(vgrad,dir);
  }
  virtual Real get_val(void) const {return val;}
  virtual Real get_grad(void) const {return grad;}
};

class FunctionAlongGreatCircle: public  Function1DWithGrad {
  Array<Real> org;
  Array<Real> dir;
  Real cdir;
  FunctionWithGrad *pfunc;
  Real val;
  Real grad;
public:
  FunctionAlongGreatCircle(const Array<Real> &_org, const Array<Real> &_dir,  FunctionWithGrad *_pfunc): Function1DWithGrad(), org(_org), dir(_dir) {
    pfunc=_pfunc;
    val=pfunc->get_val();
    Array<Real> vgrad;
    pfunc->get_grad(&vgrad);
    grad=inner_product(vgrad,dir);
    cdir=norm(org)/norm(dir);
  }
  virtual void scalar_to_vector(Array<Real> *px, Real u) {
    Array<Real> y;
    product(px,org,cos(u));
    product(&y,dir,sin(u)*cdir);
    sum(px,*px,y);
  }
  virtual void set_arg(Real u) {
    Array<Real> x;
    scalar_to_vector(&x,u);
    pfunc->set_arg(x);
    val=pfunc->get_val();
    Array<Real> vgrad;
    pfunc->get_grad(&vgrad);
    Array<Real> dx;
    scalar_to_vector(&dx,u+M_PI/2.);
    product(&dx,dx,1./cdir);
    grad=inner_product(vgrad,dx);
  }
  virtual Real get_val(void) const {return val;}
  virtual Real get_grad(void) const {return grad;}
};
/*
void dbrent(Real *px, Real ax, Real cx, const OptimizerParam &param, Function1DWithGrad *pfunc) {
  Real tiny=1e-5;
  int iter,ok1,ok2;
  Real a,b,d,d1,d2,du,dv,dw,dx,e,olde;
  Real u,u1,u2,v,w,x,xm;

  if (ax==cx) {
    *px=ax;
    return;
  }
  e=0.;
  a=(ax < cx ? ax : cx);
  b=(ax > cx ? ax : cx);
  x=w=v=(a+b)/2.;
  pfunc->set_arg(x);
  dw=dv=dx=pfunc->get_grad();
  for (iter=1; iter<=param.maxitline; iter++) {
    xm=0.5*(a+b);
    if (fabs(e) > tiny) {
      d1=2.0*(b-a);
      d2=d1;
      if (dw != dx) d1=(w-x)*dx/(dx-dw);
      if (dv != dx) d2=(v-x)*dx/(dx-dv);
      u1=x+d1;
      u2=x+d2;
      ok1 = (a-u1)*(u1-b) > 0.0 && dx*d1 <= 0.0;
      ok2 = (a-u2)*(u2-b) > 0.0 && dx*d2 <= 0.0;
      olde=e;
      e=d;
      if (ok1 || ok2) {
	if (ok1 && ok2)
	  d=(fabs(d1) < fabs(d2) ? d1 : d2);
	else if (ok1)
	  d=d1;
	else
	  d=d2;
	if (fabs(d) <= fabs(0.5*olde)) {
	  u=x+d;
	  cout << "#interp\n";
	}
	else {
	  d=0.5*(e=(dx >= 0.0 ? a-x : b-x));
	  cout << "#bisect\n";
	}
      }
      else {
	d=0.5*(e=(dx >= 0.0 ? a-x : b-x));
	cout << "#bisect\n";
      }
    }
    else {
      d=0.5*(e=(dx >= 0.0 ? a-x : b-x));
      cout << "#bisect\n";
    }
    u=x+d;
    pfunc->set_arg(u);
    du=pfunc->get_grad();
    cout << "#du= " << du << endl;
    if (fabs(du)<param.gradtol) {
      *px=u;
      return;
    }
    if (fabs(du) <= fabs(dx)) {
      if (u >= x) a=x; else b=x;
      v=w; dv=dw;
      w=x; dw=dx;
      x=u; dx=du;
    } else {
      if (u < x) a=u; else b=u;
      if (fabs(du) <= fabs(dw) || w == x) {
	v=w; dv=dw;
	w=u; dw=du;
      } else if (fabs(du) < fabs(dv) || v == x || v == w) {
	v=u; dv=du;
      }
    }
    cout << "#v= " << v << " dv= " << dv << endl;
    cout << "#x= " << x << " dx= " << dx << endl;
    cout << "#w= " << w << " dw= " << dw << endl;
  }
  ERRORQUIT("#Too many iterations in dbrent.");
  return;
}
*/
/*
void bracketmin(Real *pa, Real *pb, const OptimizerParam &param, Function1DWithGrad *pfunc) {
  *pb=*pa;
  Real dfb=pfunc->get_grad(); // at *pa;
  if (fabs(dfb)<param.gradtol) return;
  Real sdf0=sgn(dfb);
  Real dfa=0.;
  Real curstep=param.trialstep;
  for (int it=0; it<param.maxitline; it++) {
    cout << "#bracket\n";
    cout << "#a= " << *pa << " #dfa= " << dfa << endl;
    cout << "#b= " << *pb << " #dfb= " << dfb << endl;
    Real dx;
    if (fabs(dfb)<fabs(dfa)) {
      dx=min(1.5*fabs(dfb)*fabs(*pa-*pb)/(fabs(dfa)-fabs(dfb)),param.maxstep);
      cout << "#smart\n";
    }
    else {
      dx=curstep;
      curstep*=param.multline;
      curstep=min(curstep,param.maxstep);
    }
    *pa=*pb;
    dfa=dfb;
    *pb+=-dx*sdf0;
    pfunc->set_arg(*pb);
    dfb=pfunc->get_grad();
    if (dfb*sdf0<-param.gradtol) {
      cout << "#found bracket:\n";
      cout << "#a= " << *pa << " #dfa= " << dfa << endl;
      cout << "#b= " << *pb << " #dfb= " << dfb << endl;
      return;
    }
  }
  ERRORQUIT("#Too many iterations in bracketmin.");
}
*/

int min_func1d(Real *px, const OptimizerParam &param, Function1DWithGrad *pfunc) {
  Real a,b;
  b=*px;
  a=b;
  Real dfb=pfunc->get_grad(); // at *px;
  if (fabs(dfb)<param.gradtol) return 1;
  Real sdf0=sgn(dfb);
  Real dfa=0.;
  Real curstep=param.trialstep;
  int it;
  for (it=0; it<param.maxitline; it++) {
    cout << "#trialbracket\n";
    cout << "#a= " << a << " #dfa= " << dfa << endl;
    cout << "#b= " << b << " #dfx= " << dfb << endl;
    Real dx;
    if (fabs(dfb)<fabs(dfa)) {
      dx=min(1.5*fabs(dfb)*fabs(a-b)/(fabs(dfa)-fabs(dfb)),param.maxstep);
      cout << "#smart\n";
    }
    else {
      dx=curstep;
      curstep*=param.multline;
      curstep=min(curstep,param.maxstep);
    }
    a=b;
    dfa=dfb;
    b+=-dx*sdf0;
    pfunc->set_arg(b);
    dfb=pfunc->get_grad();
    if (fabs(dfb)<param.gradtol) {
      cout << "#found line min at " << b << " #dfx= " << dfb << endl;
      *px=b;
      return 1;
    }
    if (dfb*sdf0<-param.gradtol) {
      cout << "#found bracket:\n";
      cout << "#a= " << a << " #dfa= " << dfa << endl;
      cout << "#b= " << b << " #dfb= " << dfb << endl;
      break;
    }
  }
  if (it==param.maxitline) {ERRORQUIT("#Too many iterations in bracketing.");}
  int badstrike=param.badstrike;
  for (it=0; it<param.maxitline; it++) {
    Real z=dfb/(dfb-dfa);
    Real x=a*z+b*(1-z);
    if (z<0.25 || z>0.75) {
      if (badstrike>0) {
	badstrike--;
	cout << "#badstrike forgiven\n";
      }
      else {
	cout << "#midpoint" << endl;
	x=(a+b)/2.;
      }
    }
    pfunc->set_arg(x);
    Real dfx=pfunc->get_grad();
    cout << "#a= " << a << " #dfa= " << dfa << endl;
    cout << "#x= " << x << " #dfx= " << dfx << endl;
    cout << "#b= " << b << " #dfb= " << dfb << endl;
    if (fabs(dfx)<param.gradtol) {
      *px=x;
      return 1;
    }
    if (dfx*dfa<0) {
      b=x;
      dfb=dfx;
    }
    else {
      a=x;
      dfa=dfx;
    }
  }
  cout << "#Too many iterations in bisection" << endl;
  return 0;
}

void line_search(Array<Real> *px, const Array<Real> &dir, const OptimizerParam &param, FunctionWithGrad *pfunc) {
  Array<Real> ndir(dir);
  normalize(&ndir);
  Array<Real> org(*px);
  FunctionAlongLine func1d(org,ndir,pfunc);
  Real u=0.;
  min_func1d(&u,param,&func1d);
  /*
  Real a=0.;
  Real b;
  bracketmin(&a,&b,param,&func1d);
  Real u=0;
  dbrent(&u,a,b,param,&func1d);
  */
  func1d.scalar_to_vector(px,u);
}

void great_circle_search(Array<Real> *px, const Array<Real> &dir, const OptimizerParam &param, FunctionWithGrad *pfunc) {
  Array<Real> ndir(dir);
  normalize(&ndir);
  Array<Real> org(*px);
  FunctionAlongGreatCircle func1d(org,ndir,pfunc);
  Real u=0.;
  min_func1d(&u,param,&func1d);
  /*
  Real a=0.;
  Real b;
  bracketmin(&a,&b,param,&func1d);
  Real u=0;
  dbrent(&u,a,b,param,&func1d);
  */
  func1d.scalar_to_vector(px,u);
}


/*
int line_search(Array<Real> *px, const Array<Real> &dir, const OptimizerParam &param, FunctionWithGrad *pfunc) {
  //  cout << dir(0) << " " << dir(1) << endl;
  Array<Real> ndir(dir);
  normalize(&ndir);
  Real u2=0;
  Array<Real> x,g;
  pfunc->get_grad(&g);
  Real g2=inner_product(g,ndir);
  if (fabs(g2)<param.gradtol) {return 1;}
  Real u=param.trialstep;
  for (int it=0; it<param.maxitline; it++) {
    Real g1,u1;
    u1=u2;
    g1=g2;
    u2=u;
    product(&x,ndir,u2);
    sum(&x,x,*px);
    pfunc->set_arg(x);
    pfunc->get_grad(&g);
    g2=inner_product(g,ndir);
    if (fabs(g2)<param.gradtol) {
      *px=x;
      return 1;
    }
    cout << "#u12= " << u1 << " " << u2 << endl;
    //    if ((g2-g1)*(u2-u1)<0) {
    if (g1*g2>0) {
      u=u2+param.multline*fabs(u2-u1)*sgn(-g2);
      cout << "#catch line" << endl;
      if (fabs(u-u2)>param.maxstep) {
	cout << "#cap displ line" << endl;
	u=u2+param.maxstep*sgn(u-u2);
      }
    }
    else {
      u=(g2*u1-g1*u2)/(g2-g1);
    }
    cout << "#u= " << u << endl;
  }
  ERRORQUIT("#Too many iteration in line search");
}
*/

int conjugate_gradient(Array<Real> *px, const OptimizerParam &_param, FunctionWithGrad *pfunc) {
  OptimizerParam param(_param);
  pfunc->set_arg(*px);
  //  Real curf=pfunc->get_val();
  Array<Real> dir,g,h;
  pfunc->get_grad(&dir);
  product(&dir,dir,-1.);
  g=dir;
  h=dir;
  for (int it=0; it<param.maxitcg; it++) {
    Array<Real> dx(*px);
    cout << "#LS\n";
    cout << "#dir= ";
    tracev(dir);
    param.gradtol/=sqrt((Real)px->get_size());
    line_search(px,dir,param,pfunc);
    param.gradtol=_param.gradtol;
    cout << "#CG\n";
    diff(&dx,dx,*px);
    Real ndx=norm(dx);
    if (ndx!=0) {param.trialstep=MIN(ndx,param.trialstep);}
    //    if (fabs(pfunc->get_val()-curf)<tol) {return 1;}
    //    curf=pfunc->get_val();
    pfunc->get_grad(&dir);
    cout << "#l_dir= "; tracev(dir);
    cout << "#l_gnorm= " << norm(dir) << endl;
    if (norm(dir)<param.gradtol) {
      return 1;
    }
    Real g2=inner_product(g,g);
    Real dir2=inner_product(dir,dir);
    Real gam=dir2/g2;
    product(&g,dir,-1.);
    product(&h,h,gam);
    sum(&h,h,g);
    //    cout << "g= " << g(0) << " " << g(1) << endl;
    //    cout << "h= " << h(0) << " " << h(1) << endl;
    dir=h;
    //    cout << endl;
  }
  cout << "#Too many iteration in conjugate gradient" << endl;
  return 0;
}

/*
int great_circle_search(Array<Real> *px, const Array<Real> &dir, const OptimizerParam &param, FunctionWithGrad *pfunc) {
  //  cout << dir(0) << " " << dir(1) << endl;
  Array<Real> ndir(dir);
  if (normalize(&ndir)<param.gradtol) {return 1;}
  //  cout << "#n= "; tracev(dir);
  //  cout << "#dot0= " << inner_product(*px,dir) << endl;
  //  cout << "#dot0= " << inner_product(*px,ndir) << endl;
  Real u2=0;
  Array<Real> x,g;
  pfunc->get_grad(&g);
  Real g2=inner_product(g,ndir);
  Real u=param.trialstep;
  for (int it=0; it<param.maxitline; it++) {
    Real g1,u1;
    u1=u2;
    g1=g2;
    u2=u;
    Real r=norm(*px);
    Array<Real> y;
    // cout << "#dot= " << inner_product(*px,ndir) << endl;
    product(&y,*px,cos(u2));
    product(&x,ndir,r*sin(u2));
    sum(&x,x,y);

    Array<Real> ty,tdir;
    product(&ty,*px,-sin(u2)/r);
    product(&tdir,ndir,cos(u2));
    sum(&tdir,tdir,ty);
    pfunc->set_arg(x);
    cout << "#x= "; tracev(x);
    pfunc->get_grad(&g);
    g2=inner_product(g,tdir);
    cout << "#u= " << u2 << endl;
    cout << "#e= " << pfunc->get_val() << endl;
    cout << "#g= " << g2 << endl;
    // cout << "#|x|= " << norm(x) << endl;
    if (fabs(g2)<param.gradtol) {
      *px=x;
      cout << "#fgc\n";
      return 1;
    }
    //    if ((g2-g1)*(u2-u1)<0) {
    if (g1*g2>0) {
      u=u2+param.multline*fabs(u2-u1)*sgn(-g2);
      cout << "#catch sph" << endl;
      if (fabs(u-u2)>param.maxstep) {
	cout << "#maxtep" << endl;
	u=u2+param.maxstep*sgn(u-u2);
      }
    }
    else {
      u=(g2*u1-g1*u2)/(g2-g1);
    }
  }
  ERRORQUIT("#Too many iteration in great circle search");
}
*/

void ortho_project(Array<Real> *px, const Array<Real> &p, const Array<Real> &x) {
  Real a=inner_product(p,x)/inner_product(p,p);
  Array<Real> ap;
  product(&ap,p,a);
  diff(px,x,ap); //*px=x-p <p|x>/<p|p)>;
}

int conjugate_gradient_on_sphere(Array<Real> *px, const OptimizerParam &_param, FunctionWithGrad *pfunc) {
  OptimizerParam param(_param);
  pfunc->set_arg(*px);
  //  Real curf=pfunc->get_val();
  Array<Real> dir,g,h;
  pfunc->get_grad(&dir);
  ortho_project(&dir,*px,dir);
  product(&dir,dir,-1.);
  g=dir;
  h=dir;
  for (int it=0; it<param.maxitcg; it++) {
    Array<Real> dx(*px);
    param.gradtol/=sqrt((Real)px->get_size());
    great_circle_search(px,dir,param,pfunc);
    param.gradtol=_param.gradtol;
    diff(&dx,dx,*px);
    Real ndx=norm(dx);
    if (ndx!=0) {param.trialstep=MIN(ndx/norm(*px),param.trialstep);}
    //    if (fabs(pfunc->get_val()-curf)<tol) {return 1;}
    //    curf=pfunc->get_val();
    pfunc->get_grad(&dir);
    ortho_project(&dir,*px,dir);
    cout << "#s_dir= "; tracev(dir);
    cout << "#s_gnorm= " << norm(dir) << endl;
    if (norm(dir)<param.gradtol) {return 1;}
    if (norm(dx)>norm(*px)) {
      product(&dir,dir,-1.);
      g=dir;
      h=dir;
    }
    else {
      ortho_project(&g,*px,g);
      ortho_project(&h,*px,h);
      Real g2=inner_product(g,g);
      Real dir2=inner_product(dir,dir);
      Real gam=dir2/g2;
      product(&g,dir,-1.);
      product(&h,h,gam);
      sum(&h,h,g);
    }
    cout << "#gv= "; tracev(g);
    cout << "#hv= "; tracev(h);
    ortho_project(&h,*px,h);
    dir=h;
    cout << "#sCG" << endl;
    //    cout << endl;
    cout << "#it= " << it << endl;
  }
  cout << "#Too many iteration in conjugate gradient on sphere" << endl;
  return 0;
}

class SmallEpicycle: public FunctionWithGrad {
  FunctionWithGrad *pfunc;
  Array<Real> c;
  Array<Real> s;
  Array<Real> g0;
  Array<Real> df;
  Real e;
public:
  SmallEpicycle(FunctionWithGrad *_pfunc): FunctionWithGrad(), c(), s(), g0(), df() {pfunc=_pfunc;}
  void init(const Array<Real> _c) {
    c=_c;
    pfunc->set_arg(c);
    pfunc->get_grad(&g0);
  }
  void get_grad_org(Array<Real> *pg) {*pg=g0;}
  void set_arg(const Array<Real> &_s) {
    cout << "#Small" << endl;
    Array<Real> x;
    s=_s;
    sum(&x,c,s);
    //    tracev(c);
    // tracev(s);
    // cout << endl << endl;
    pfunc->set_arg(x);
    e=pfunc->get_val();
    Array<Real> g;
    pfunc->get_grad(&g);
    diff(&df,g,g0);
    product(&df,df,1./norm(s));
  }
  Real get_val(void) const {return e;}
  void get_grad(Array<Real> *pdf) const {*pdf=df;}
};

class MotionEpicycle: public FunctionWithGrad {
  FunctionWithGrad *pfunc;
  OptimizerParam sparam;
  Real slen;
  Real constif;
  Array<Real> s0;
  Real energy;
  Array<Real> gradenergy;
public:
  MotionEpicycle(FunctionWithGrad *_pfunc): FunctionWithGrad(), sparam(), s0(), gradenergy() {pfunc=_pfunc;}
  void init(const OptimizerParam &_sparam, Real _slen, Real _constif) {
    sparam=_sparam;
    slen=_slen;
    constif=_constif;
  }
  void set_arg(const Array<Real> &c) {
    cout << "#Motion" << endl;
    SmallEpicycle sepi(pfunc);
    cout << "#c= "; tracev(c);
    sepi.init(c);
    {
      ofstream file("epipos.out");
      file << c;
    }
    system("cp str.out str_current.out");
    energy=pfunc->get_val();
    Array<Real> f0;
    pfunc->get_grad(&f0);
    // sepi.get_grad_org(&f0);
    cout << "#f0= "; tracev(f0);
    cout << "#s0= "; tracev(s0);
    if (s0.get_size()==0) {
      ifstream file("epidir.out");
      if (file) {
	file >> s0;
      }
      else {
	product(&s0,f0,-slen/norm(f0));
	for (int i=0; i<s0.get_size(); i++) {
	  s0(i)+=(2*uniform01()-1.)*sparam.trialstep*slen;
	}
      }
      product(&s0,s0,slen/norm(s0));
    }
    conjugate_gradient_on_sphere(&s0,sparam,&sepi);
    {
      ofstream file("epidir.out");
      file << s0;
    }

    Array<Real> f1,f2;
    Real e1,e2;
    Array<Real> cs;

    sum(&cs,c,s0); tracevl(cs);
    //pfunc->set_arg(cs);
    pfunc->get_grad(&f1);
    e1=pfunc->get_val();
    cout << e1 << endl;

    diff(&cs,c,s0);
    pfunc->set_arg(cs);
    pfunc->get_grad(&f2);
    e2=pfunc->get_val();
    tracevl(cs);
    cout << e2 << endl << endl << endl;
    cout << "#f0= "; tracev(f0);
    cout << "#f1= "; tracev(f1);
    cout << "#f2= "; tracev(f2);
    Real s2=inner_product(s0,s0);
    Real mcurv=-(e1+e2-2.*energy)/s2;
    cout << "#mcurv= " << mcurv << endl;
    cout << "#energy= " << energy << endl;
    Array<Real> gradmcurv;
    product(&gradmcurv,f0,-2.);
    sum(&gradmcurv,gradmcurv,f1);
    sum(&gradmcurv,gradmcurv,f2);
    product(&gradmcurv,gradmcurv,-1./s2);

    {
      Array<Real> tmp;
      tracevl(c);
      cout << energy << endl;
      product(&tmp,gradmcurv,-slen/norm(gradmcurv));
      sum(&tmp,c,tmp);
      tracevl(tmp);
      cout << energy << endl << endl << endl;
    }

    Array<Real> fperp;
    ortho_project(&fperp,gradmcurv,f0);
    
    Real zerocurv=-mcurv/norm(gradmcurv);
    if (constif==0.) {
      constif=norm(f0)/max(fabs(zerocurv),0.1*sparam.trialstep);
      cout << "#constif= " << constif << endl;
    }
    Array<Real> fpar;
    
    product(&fpar,gradmcurv,-zerocurv*constif/norm(gradmcurv));

    sum(&gradenergy,fpar,fperp);
    cout << "#zer= " << zerocurv << endl;
    cout << "#forpar= "; tracev(fpar);
    cout << "#forperp= "; tracev(fperp);
    cout << "#for= "; tracev(gradenergy);
    cout << "#fornorm par= " << norm(fpar) << " perp= " << norm(fperp) << endl;
  }
  Real get_val(void) const {return energy;}
  void get_grad(Array<Real> *df) const {*df=gradenergy;}
};

int steepest_descent(Array<Real> *px, const OptimizerParam &param, FunctionWithGrad *pfunc) {
  for (int it=0; it<param.maxitcg; it++) {
    Array<Real> g;
    pfunc->set_arg(*px);
    pfunc->get_grad(&g);
    if (norm(g)<param.gradtol) {return 1;}
    product(&g,g,-param.trialstep);
    sum(px,*px,g);
  }
  return 0;
}

Real find_min_in_unstable(Array<Real> *pc, const OptimizerParam &sparam, Real slen, const OptimizerParam &steepparam, const OptimizerParam &ionparam, Real constif, FunctionWithGrad *pfunc) {
  MotionEpicycle mepi(pfunc);
  mepi.init(sparam,slen,constif);
  if (steepest_descent(pc,steepparam,&mepi)) {return mepi.get_val();}
  cout << "#STARTCG\n";
  conjugate_gradient(pc,ionparam,&mepi);
  return mepi.get_val();
}

Real myfr(Real r) {
  return ipow(r,2)/2.-ipow(r,3)/6.;
}

Real myf(Real x, Real y) {
  //  return pow(1+cos(x),1.5)+pow(1+cos(y),1.5)+sqr(z);
  //  return myfr(sqrt(sqr(x)+sqr(y)))+1.0*sqr(y)+x/8.+3.*exp(-sqr(x-2.3)-sqr(y-0.5))+sqr(x)+0.2*y;
  return myfr(sqrt(sqr(x)+sqr(y)))+1.0*sqr(y)+x/8.+3.*exp(-sqr(x-2.3)-sqr(0.75*(y-0.5)))+sqr(x)+0.2*y;
}

class MyFunc: public FunctionWithGrad {
  Array<Real> x;
  Array<Real> df;
public:
  MyFunc(void): FunctionWithGrad(), x() {}
  void set_arg(const Array<Real> &_x) {
    x=_x;
    df.resize(x.get_size());
    Real dx=0.01;
    df(0)=(myf(x(0)+dx,x(1))-myf(x(0)-dx,x(1)))/(2.*dx);
    df(1)=(myf(x(0),x(1)+dx)-myf(x(0),x(1)-dx))/(2.*dx);
    //cout << "#gnorm= " << norm(df) << endl;
}
  Real get_val(void) const {return myf(x(0),x(1));}
  void get_grad(Array<Real> *pdf) const {
    *pdf=df;
  }
};

void Array6_to_Matrix3d(rMatrix3d *pm, const Array<Real> &x, int do2d=0) {
  if (!do2d) {
    for (int i=0; i<3; i++) {
      for (int j=0; j<3; j++) {
	int ii=i;
	int jj=j;
	if (j<i) {swap(&ii,&jj);}
	(*pm)(i,j)=x((2*3+1-ii)*ii/2+jj-ii);
      }
    }
  }
  else {
    for (int i=0; i<3; i++) {
      for (int j=0; j<3; j++) {
	if (i<2 && j<2) {
	  int ii=i;
	  int jj=j;
	  if (j<i) {swap(&ii,&jj);}
	  (*pm)(i,j)=x(ii+jj);
	}
	else {
	  (*pm)(i,j)=(i==j ? 1.: 0.);
	}
      }
    }
  }
}

void Matrix3d_to_Array6(Array<Real> *px, const rMatrix3d &m, int do2d=0) {
  if (!do2d) {
    if (px->get_size()==0) {px->resize(6);}
    int k=0;
    for (int i=0; i<3; i++) {
      for (int j=i; j<3; j++) {
	(*px)(k)=m(i,j);
	k++;
      }
    }
  }
  else {
    if (px->get_size()==0) {px->resize(3);}
    int k=0;
    for (int i=0; i<2; i++) {
      for (int j=i; j<2; j++) {
	(*px)(k)=m(i,j);
	k++;
      }
    }
  }
}

class OptimizedStructure: public FunctionWithGrad {
  Real val;
  Array<Real> grad;
  rVector3d sumpos;
  Real forcescale;
  int do2d;
public:
  rMatrix3d axes;
  Structure str;
  Array<AutoString> atom_label;
  int sleeptime;
public:
  OptimizedStructure(void): FunctionWithGrad(), grad(), val(0), axes(), str(), atom_label(), sumpos() {forcescale=1; sleeptime=5; do2d=0;}
  ~OptimizedStructure(void) {ofstream file("stop");}
  void init(Array<Real> *px, const rMatrix3d &_axes, const Structure &_str, const Array<AutoString> &_atom_label, Real forcefact=1., int _do2d=0) {
    axes=_axes;
    str=_str;
    do2d=_do2d;
    forcescale=pow((Real)(str.atom_pos.get_size()),1./3.)*forcefact;
    atom_label=_atom_label;
    px->resize((do2d ? 3 : 6)+3*(str.atom_pos.get_size()-1));
    rMatrix3d id;
    id.identity();
    Matrix3d_to_Array6(px,id,do2d);
    sumpos=rVector3d(0.,0.,0.);
    rMatrix3d icell=!str.cell;
    for (int at=0; at<str.atom_pos.get_size(); at++) {
      rVector3d v=icell*str.atom_pos(at)*forcescale;
      sumpos+=v;
      if (at>0) {
	for (int j=0; j<3; j++) {
	  (*px)((do2d ? 3 : 6)+3*(at-1)+j)=v(j);
	}
      }
    }
  }
  void vect_to_str(Structure *pstr, const Array<Real> &x) {
    pstr->atom_type=str.atom_type;
    rMatrix3d strainI;
    Array6_to_Matrix3d(&strainI,x,do2d);
    pstr->cell=strainI*str.cell;
    pstr->atom_pos.resize(str.atom_pos.get_size());
    rVector3d v0(0.,0.,0.);
    for (int at=1; at<pstr->atom_pos.get_size(); at++) {
      rVector3d v;
      for (int j=0; j<3; j++) {
	v(j)=x((do2d ? 3 : 6)+3*(at-1)+j);
      }
      pstr->atom_pos(at)=pstr->cell*v*(1./forcescale);
      v0+=v;
    }
    pstr->atom_pos(0)=pstr->cell*(sumpos-v0)*(1./forcescale);
  }
  void set_arg(const Array<Real> &x) {
    Real stressconv=1e8/1.6e-19*1e-30; // kB to eV/A^3;
    Structure curstr;
    cout << "#x= ";
    tracev(x);
    vect_to_str(&curstr,x);
    {
      ofstream file("str.out");
      file.setf(ios::fixed);
      file.precision(5);
      write_structure(curstr,atom_label,axes,file,0);
    }
    {ofstream flag("busy");}
    while (file_exists("busy")) {
      sleep(sleeptime);//
    }
    {
      ifstream efile("energy");
      efile >> val;
    }
    grad.resize((do2d ? 3 : 6)+3*(str.atom_pos.get_size()-1));
    {
      ifstream sfile("stress.out");
      rMatrix3d rawstress;
      rawstress(2,2)=MAXFLOAT;
      for (int i=0; i<3; i++) {
	for (int j=0; j<3; j++) {
	  sfile >> rawstress(i,j);
	}
      }
      if (rawstress(2,2)==MAXFLOAT) {
	ERRORQUIT("Error reading stress.out file.");
      }
      rMatrix3d strainI,stress;
      Array6_to_Matrix3d(&strainI,x,do2d);
      stress=-stressconv*det(curstr.cell)/(Real)(str.atom_pos.get_size())*rawstress*(~ ! strainI);
      Matrix3d_to_Array6(&grad,stress,do2d);
    }
    {
      ifstream ffile("force.out");
      Array<rVector3d> rawforce(str.atom_pos.get_size());
      rawforce(rawforce.get_size()-1)(2)=MAXFLOAT;
      for (int i=0; i<rawforce.get_size(); i++) {
	ffile >> rawforce(i);
      }
      if (rawforce(rawforce.get_size()-1)(2)==MAXFLOAT) {
	ERRORQUIT("Error reading force.out file.");
      }

      for (int at=1; at<rawforce.get_size(); at++) {
	rVector3d force=(~ curstr.cell)*(rawforce(at)-rawforce(0))*(1./forcescale);
	for (int j=0; j<3; j++) {
	  grad((do2d ? 3 : 6)+3*(at-1)+j)=-force(j);
	}
      }
    }
  }
  Real get_val(void) const {return val;}
  void get_grad(Array<Real> *pgrad) const {*pgrad=grad;}  
};

class MyFunk: public Function1DWithGrad {
  Real x;
  Real f(Real y) const {return -8./(1.+ipow(y-1,8))-y*exp(-0.5*y);}
public:
  void set_arg(Real _x) {x=_x; cout << x << " " << f(x) << endl;}
  Real get_val(void) const {return f(x);}
  Real get_grad(void) const {Real dx=1e-3; return (f(x+dx)-f(x-dx))/(2.*dx);}
  
};


int main(int argc, char *argv[]) {
  
  int sigdig=5;
  OptimizerParam sparam,steepparam,ionparam;
  sparam.gradtol=1.0;
  sparam.trialstep=0.1;
  sparam.maxstep=M_PI/4;
  sparam.multline=2.0;
  sparam.maxitline=40;
  sparam.maxitcg=40;
  sparam.badstrike=1;
  Real slen=0.01;

  steepparam.gradtol=1.0;
  steepparam.trialstep=0.01;
  steepparam.maxitcg=0;

  ionparam.gradtol=1.0;
  ionparam.trialstep=0.003;
  ionparam.maxstep=0.01;
  ionparam.multline=2.0;
  ionparam.maxitline=40;
  ionparam.maxitcg=40;
  ionparam.badstrike=1;
  Real constif=0.0;
  Real forcefact=4.;
  int do2d=0;
  int sleeptime=5;

  int relax=0;
  int debug=0;
  int analex=0;
  int defaults=0;
  Array<Real> tmpparam(2);
  tmpparam(0)=1.9;
  tmpparam(1)=-02.;

  AskStruct options[]={
    {"","INFlection DETection method " MAPS_VERSION ", by Axel van de Walle",TITLEVAL,NULL},
    {"-egt","Epicycle Gradient Tolerance",REALVAL,&sparam.gradtol},
    {"-ets","Epicycle Trial Step",REALVAL,&sparam.trialstep},
    {"-ems","Epicycle Maximum Step",REALVAL,&sparam.maxstep},
    {"-eml","Epicycle Multiplier in Line minimization",REALVAL,&sparam.multline},
    {"-eil","Epicycle maximum number of Iterations in Line minimizations",INTVAL,&sparam.maxitline},
    {"-eic","Epicycle maximum number of Iterations in Conjugate gradient",INTVAL,&sparam.maxitcg},
    {"-ebf","Epicycle motion Bad step Forgiveness",INTVAL,&sparam.badstrike},
    {"-el","Epicycle Length",REALVAL,&slen},

    {"-ism","Ion motion Steepest descent Multiplier",REALVAL,&steepparam.trialstep},
    {"-isi","Ion motion Steepest maximum number of Iterations",INTVAL,&steepparam.maxitcg},
    {"-igt","Ion motion Gradient Tolerance",REALVAL,&ionparam.gradtol},
    {"-its","Ion motion Trial Step",REALVAL,&ionparam.trialstep},
    {"-ims","Ion motion Maximum Step",REALVAL,&ionparam.maxstep},
    {"-iml","Ion motion Multiplier in Line minimization",REALVAL,&ionparam.multline},
    {"-iil","Ion motion maximum number of Iterations in Line minimizations",INTVAL,&ionparam.maxitline},
    {"-iic","Ion motion maximum number of Iterations in Conjugate gradient",INTVAL,&ionparam.maxitcg},
    {"-ibf","Ion motion Bad step Forgiveness",INTVAL,&ionparam.badstrike},
    {"-ics","Ion motion Curvature constraint Stiffness (0=automatic)",REALVAL,&constif},
    {"-ff","Force scale Factor",REALVAL,&forcefact},
    {"-2d","Considers only 2d strain and stress",BOOLVAL,&do2d},
    {"-st","Sleep Time between read access",INTVAL,&sleeptime},
    {"-r","Relax to minimum (for debugging only)",BOOLVAL,&relax},
    {"-db","debug",BOOLVAL,&debug},
    {"-aex","Analytical Example (for debugging)",BOOLVAL,&analex},
    {"-t1","Temp param 1",REALVAL,&tmpparam(0)},
    {"-t2","Temp param 2",REALVAL,&tmpparam(1)},
    {"-d","Use all default values",BOOLVAL,&defaults}
  };
  if (!get_values(argc,argv,countof(options),options)) {
    display_help(countof(options),options);
    return 1;
  }

  steepparam.gradtol=ionparam.gradtol;


  if (analex) {
    {
      Real dx=0.01;
      ofstream file("ftmp.out");
      Array<Real> x(2);
      for (x(0)=-2.; x(0)<=2.; x(0)+=0.05) {
	for (x(1)=-2.; x(1)<=2.; x(1)+=0.05) {
	  Real gxx=(myf(x(0)+dx,x(1))+myf(x(0)-dx,x(1))-2.*myf(x(0),x(1)))/sqr(dx);
	  Real gyy=(myf(x(0),x(1)+dx)+myf(x(0),x(1)-dx)-2.*myf(x(0),x(1)))/sqr(dx);
	  Real gxy=(myf(x(0)+dx,x(1)+dx)-myf(x(0)+dx,x(1)-dx) - ( myf(x(0)-dx,x(1)+dx)-myf(x(0)-dx,x(1)-dx) ) )/sqr(dx)/4.0;
	  Real t=(gxx+gyy)/2.;
	  Real d=gxx*gyy-gxy*gxy; // 1*x^2-2*t*x+d;
	  Real mincurv=t-sqrt(sqr(t)-d);
	  file << x(0) << " " << x(1) << " " << myf(x(0),x(1)) << " " << mincurv << endl;
	}
	file << endl;
      }
    }

    MyFunc f;
    Array<Real> x(2);
    //  x(0)=1.2; x(1)=0.5;
    //    x(0)=1.9; x(1)=-0.2;
    x=tmpparam;

    Real e=find_min_in_unstable(&x,sparam,slen,steepparam,ionparam,constif,&f);
    exit(0);
  }


  rMatrix3d axes;
  Structure str;
  Array<AutoString> atom_label;
  {
    Array<Array<int> > site_type_list;
    ifstream file("str.in");
    parse_lattice_file(&str.cell,&str.atom_pos,&str.atom_type,&site_type_list,&atom_label,file,&axes);
    fix_atom_type(&str,site_type_list);
  }

  OptimizedStructure optstr;
  Array<Real> x;
  optstr.init(&x,axes,str,atom_label,forcefact,do2d);
  optstr.sleeptime=sleeptime;

  Real e;
  if (relax==1) {
    e=conjugate_gradient(&x,ionparam,&optstr);
  }
  else if (debug==1) {
    MotionEpicycle epi(&optstr);
    epi.init(sparam, slen, constif);
    for (int i=0; i<ionparam.maxitline; i++) {
      epi.set_arg(x);
      x(ionparam.maxitcg)+=ionparam.trialstep;
    }
  }
  else {
    ifstream file("epipos.out");
    if (file) {
      file >> x;
    }
    e=find_min_in_unstable(&x, sparam,slen,steepparam,ionparam,constif,&optstr);
  }

  {
    ofstream ofile("cenergy.out");
    ofile.setf(ios::fixed);
    ofile.precision(sigdig);
    ofile << e << endl;
  }

  Structure relstr;
  optstr.vect_to_str(&relstr,x);
  {
    ofstream ofile("cstr_relax.out");
    ofile.setf(ios::fixed);
    ofile.precision(sigdig);
    write_structure(relstr,atom_label,axes,ofile,0);
  }
  
  /*  
  MyFunc f;

  OptimizerParam ionparam;
  ionparam.gradtol=1e-4;
  ionparam.trialstep=0.02;
  ionparam.maxstep=1.0;
  ionparam.multline=2.0;
  ionparam.maxitline=20;
  ionparam.maxitcg=40;

  Array<Real> x(2);
  x(0)=1.8; x(1)=-0.3;

  conjugate_gradient(&x,ionparam,&f);
  */

/*
  OptimizerParam param;  
  param.gradtol=1e-3;
  param.trialstep=0.02;
  param.maxstep=0.3;
  param.multline=2.0;
  param.maxitline=20;
  MyFunk func;
  Real x=0;
  func.set_arg(x);
  min_func1d(&x,param,&func);
  cout << "Min= " << x << endl;
*/
}
